==> filament-sink-websocket-server-0.0.9.tgz: package/sinks.js <==
'use strict';
//templater for sink commands
var path = require("path")
var fs = require("fs");
var nmea = require('nmea');
var EventEmitter2 = require("eventemitter2").EventEmitter2;
function guid() {
  function s4() {
    return Math.floor((1 + Math.random()) * 0x10000)
      .toString(16)
      .substring(1);
  }
  return s4() + s4() + '-' + s4() + '-' + s4() + '-' +
    s4() + '-' + s4() + s4() + s4();
}


const merger = (prefix, thing) => (gather, key) => {
  gather[prefix+key] = Array.isArray(thing) ? thing[idx] : thing[key];
  return gather;
}

const expandCDC = module.exports = (obj) => {
  try {
    if (obj.usb_cdc.indexOf("GPGGA") >= 0){
      var gps = nmea.parse(obj.usb_cdc);
      console.log("got gps", gps)
      return Object.keys(gps).reduce(merger("gps_", gps), obj)
    }
    switch(obj.usb_cdc.split(",")[0]){
      case "getCurrent":
        (["time","bus_voltage","shunt_voltage","load_voltage","current","amp_hours","watt_hours"]).reduce(merger("current_",obj.usb_cdc.split(',').slice(1)) ,obj)
      case "peopleCounter":
        (["human_count","distance"]).reduce(merger("people_", obj.usb_cdc.split(',').slice(1)), obj)
      default :
        return obj
    }
  } catch (e) {
    console.log(e);
    return obj;
  }
}
//seperate

// turn "-i,--inline <stuff>" => "inline"
const optkey = (opt) => opt[0].split(",--")[1].split(" ")[0];

const wildcard = (events, label) => (events.indexOf("*") > -1 && events.indexOf("-" + label) < 0)

const explicit = (events, label) => (events.indexOf(label) > -1)

const eventEnabled = (events, label) => (wildcard(events, label) || explicit(events,label))

const flattenReport = (obj) => Object.keys(obj).reduce((nobj, key) => {
  if (key == "usb"){
    Object.keys(obj[key]).forEach((kk) => {
      nobj["usb_" + kk] = obj[key][kk];
    })
  } else {
    nobj[key === "hn" ? "id" : key] = obj[key]
    if (typeof nobj[key] == 'string'){
       nobj[key] = nobj[key]
    }
  }
  return nobj
}, {})

const unpackReport = (array) => {
  if (typeof array[0] === "object") return array.map(obj => {
    if (obj.rssi){
      let rssi = obj.rssi.split(",").map((val) => Number.parseInt(val))
      obj.rssi_last = rssi[0];
      obj.rssi_best = rssi[1];
      obj.rssi_worst = rssi[2];
    }
    if (obj.stats){
      let stats = obj.stats.split(",").map(val => Number.parseInt(val))
      obj.rx = stats[0];
      obj.bad = stats[1];
      obj.miss = stats[2];
      obj.idle = stats[3];
    }
    return obj;
  });

  try {
    let keys = array[1][0] === "v" ? ["hn","build","error","warn","up","mem","tup","tmem", "links"] : ["hn","up","temp","humidity","lux","movex","movey","movez","dB","rssi","usb"];
    return keys.reduce((rep, key, index) => {
      if (key == "movex")
        rep["move"] = array.slice(index, index + 3).join(',');
      else if (key == "usb")
        rep["usb"] = {cdc : array[index] || undefined}
      else if (key == "movey" || key == "movez")
        return rep;
      else if (key == "links")
        rep["links"] = array.slice(index);
      else
        rep[key] = array[index];
      return rep;
    }, {});
  } catch (E) {
    return array;
  }
}

const parseJSON = (raw) => (new Promise((res,rej) => {
  res(JSON.parse(raw))
})).catch(() => {
  return JSON.parse(raw.split("}{").join("},{"));
}).catch(() => {
  return typeof raw === "string" ? raw.split(",") : raw
})

const normalize = (evt) => {
  return parseJSON(evt.data).then((data) => {
    if (Array.isArray(data))
      data = unpackReport(data);
    if (typeof data == "object" && !Array.isArray(data))
      data = flattenReport(data);
    if (data.usb_cdc)
      data = expandCDC(data);

    evt.data = data;
    if (typeof evt.data === "object" && evt.data.id){
      evt.id = evt.data.id;
    }
    return evt;
  })
}

const invalid = (args, options) =>
  options.reduce((res, opt) =>
    typeof res == "string"       ? res
    : !args.options[optkey(opt)] && !opt[2] && (typeof opt[2] != "string") ? "missing option " + optkey(opt)
    : false
  , false);



const commandTemplate = module.exports = (p) => (args) => (Mesh, th) => {
  try {
  if (!Mesh._sink_events) {
    Mesh._sink_events = new EventEmitter2();
    Mesh.on('open' , (link, open) => {
      if (open && open.json && open.json.type == "event"){
        var bodydata;
        try {
          bodydata = JSON.parse(open.body.toString().trim());
          bodydata = JSON.parse(bodydata);
        } catch (e){
          //console.log(bodydata);
        }

        Mesh._sink_events.emit(open.json.label, Object.assign({hashname: link.hashname, data : bodydata},open.json))
      }
    })
  }
  require(path.join(__dirname, p))((name, options, setup) => {
    args = args || {};
    args.options = args.options || {};
    let error = invalid(args, options);
    if (error) throw new Error(error);


    options.forEach((opt) => {
      if (opt[2] && !args.options[optkey(opt)]) args.options[optkey(opt)] = opt[2];
    })

    var events = (args.events || "*").split(",")
    setup(args.options, (runner) => {
      Mesh._sink_events.onAny((label, evt) => {
        try{
          if (eventEnabled(events, label)){
            normalize(evt).then((evt) => {
              if (!evt.id) evt.id = evt.hashname ? evt.hashname.substr(0,8) : evt.hn ? evt.hn : null;
              evt.guid = guid();
              runner(label, evt);
            })

          }
        } catch (e){
          console.log(`error in ${name} sink`,e);
        }
      })
    })
  })
} catch (e){
  console.error(e);
} finally {
  return {};
}

} ;

==> filament-sink-websocket-server-0.0.10.tgz: package/sinks.js <==
'use strict';
//templater for sink commands
var path = require("path")
var fs = require("fs");
var nmea = require('nmea');
var EventEmitter2 = require("eventemitter2").EventEmitter2;
function guid() {
  function s4() {
    return Math.floor((1 + Math.random()) * 0x10000)
      .toString(16)
      .substring(1);
  }
  return s4() + s4() + '-' + s4() + '-' + s4() + '-' +
    s4() + '-' + s4() + s4() + s4();
}


const merger = (prefix, thing) => (gather, key) => {
  gather[prefix+key] = Array.isArray(thing) ? thing[idx] : thing[key];
  return gather;
}

const expandCDC = module.exports = (obj) => {
  try {
    if (obj.usb_cdc.indexOf("GPGGA") >= 0){
      var gps = nmea.parse(obj.usb_cdc);
      console.log("got gps", gps)
      return Object.keys(gps).reduce(merger("gps_", gps), obj)
    }
    switch(obj.usb_cdc.split(",")[0]){
      case "getCurrent":
        (["time","bus_voltage","shunt_voltage","load_voltage","current","amp_hours","watt_hours"]).reduce(merger("current_",obj.usb_cdc.split(',').slice(1)) ,obj)
      case "peopleCounter":
        (["human_count","distance"]).reduce(merger("people_", obj.usb_cdc.split(',').slice(1)), obj)
      default :
        return obj
    }
  } catch (e) {
    console.log(e);
    return obj;
  }
}
//seperate

// turn "-i,--inline <stuff>" => "inline"
const optkey = (opt) => opt[0].split(",--")[1].split(" ")[0];

const wildcard = (events, label) => (events.indexOf("*") > -1 && events.indexOf("-" + label) < 0)

const explicit = (events, label) => (events.indexOf(label) > -1)

const eventEnabled = (events, label) => (wildcard(events, label) || explicit(events,label))

const flattenReport = (obj) => Object.keys(obj).reduce((nobj, key) => {
  if (key == "usb"){
    Object.keys(obj[key]).forEach((kk) => {
      nobj["usb_" + kk] = obj[key][kk];
    })
  } else {
    nobj[key === "hn" ? "id" : key] = obj[key]
    if (typeof nobj[key] == 'string'){
       nobj[key] = nobj[key]
    }
  }
  return nobj
}, {})

const unpackReport = (array, label) => {
  if (typeof array[0] === "object") return array.map(obj => {
    if (obj.rssi){
      let rssi = obj.rssi.split(",").map((val) => Number.parseInt(val))
      obj.rssi_last = rssi[0];
      obj.rssi_best = rssi[1];
      obj.rssi_worst = rssi[2];
    }
    if (obj.stats){
      let stats = obj.stats.split(",").map(val => Number.parseInt(val))
      obj.rx = stats[0];
      obj.bad = stats[1];
      obj.miss = stats[2];
      obj.idle = stats[3];
    }
    return obj;
  });

  try {
    if (label == "giga"){
      var keys = ["up","temp","humidity","lux","movex","movey","movez","dB","rssi"];
    } else {
      var keys = array[1][0] === "v" ? ["hn","build","error","warn","up","mem","tup","tmem", "links"] : ["hn","up","temp","humidity","lux","movex","movey","movez","dB","rssi","usb"];
    }
    return keys.reduce((rep, key, index) => {
      if (key == "movex")
        rep["move"] = array.slice(index, index + 3).join(',');
      else if (key == "usb")
        rep["usb"] = {cdc : array[index] || undefined}
      else if (key == "movey" || key == "movez")
        return rep;
      else if (key == "links")
        rep["links"] = array.slice(index);
      else
        rep[key] = array[index];
      return rep;
    }, {});
  } catch (E) {
    console.log("ERROR UNPACKING",E);
    return array;
  }
}

const parseJSON = (raw) => (new Promise((res,rej) => {
  res(JSON.parse(raw))
})).catch(() => {
  return JSON.parse(raw.split("}{").join("},{"));
}).catch(() => {
  return typeof raw === "string" ? raw.split(",") : raw
})

const normalize = (evt, label) => {
  return parseJSON(evt.data).then((data) => {
    if (Array.isArray(data))
      data = unpackReport(data, label);
    if (typeof data == "object" && !Array.isArray(data))
      data = flattenReport(data);
    if (data.usb_cdc)
      data = expandCDC(data);

    evt.data = data;
    if (typeof evt.data === "object" && evt.data.id){
      evt.id = evt.data.id;
    }
    return evt;
  })
}

const invalid = (args, options) =>
  options.reduce((res, opt) =>
    typeof res == "string"       ? res
    : !args.options[optkey(opt)] && !opt[2] && (typeof opt[2] != "string") ? "missing option " + optkey(opt)
    : false
  , false);



const commandTemplate = module.exports = (p) => (args) => (Mesh, th) => {
  try {
  if (!Mesh._sink_events) {
    Mesh._sink_events = new EventEmitter2();
    Mesh.on('open' , (link, open) => {
      if (open && open.json && open.json.type == "event"){
        var bodydata;
        try {
          bodydata = JSON.parse(open.body.toString().trim());
          bodydata = JSON.parse(bodydata);
        } catch (e){
          //console.log(bodydata);
        }

        Mesh._sink_events.emit(open.json.label, Object.assign({hashname: link.hashname, data : bodydata},open.json))
      }
    })
  }
  require(path.join(__dirname, p))((name, options, setup) => {
    args = args || {};
    args.options = args.options || {};
    let error = invalid(args, options);
    if (error) throw new Error(error);


    options.forEach((opt) => {
      if (opt[2] && !args.options[optkey(opt)]) args.options[optkey(opt)] = opt[2];
    })

    var events = (args.events || "*").split(",")
    setup(args.options, (runner) => {
      Mesh._sink_events.onAny((label, evt) => {
        try{
          if (eventEnabled(events, label)){
            normalize(evt, label).then((evt) => {
              if (!evt.id) evt.id = evt.hashname ? evt.hashname.substr(0,8) : evt.hn ? evt.hn : null;
              evt.guid = guid();
              runner(label, evt);
            })
            .catch((e) => {
              console.log("error in sink",e);
            })

          }
        } catch (e){
          console.log(`error in ${name} sink`,e);
        }
      })
    })
  })
} catch (e){
  console.error(e);
} finally {
  return {};
}

} ;
